//---------------------------------------------------------------------------
//
#include "Utils.hpp"
//
//---------------------------------------------------------------------------
// Rechercher tous les noeuds
void FindNodes(CNode **Nodes, CNode *Node)
{
	if (Node->Size > 1)
	{
		Nodes[Node->Num] = Node;

		FindNodes(Nodes, Node->L);
		FindNodes(Nodes, Node->R);
	}
}
//---------------------------------------------------------------------------
// If is a subfamily node
bool SubGroupNode(CNode *N, float T)
{
	if (N->Size > 1)
	{
		if (N->Coss > T) return false;
		if (!SubGroupNode(N->L, T) || !SubGroupNode(N->R, T)) return false;
	}

	return true;
}
//---------------------------------------------------------------------------
// Regression lineaire sur les mesures (renvoie le coefficient)
float LineairRegression(float **mesures, int n)
{
	int   i;
	float centerx, centery, covariance, variancex;

	// Init centers
	centerx = 0;
	centery = 0;

	for(i=0; i<n; i++)
	{
		centerx += mesures[i][0];
		centery += mesures[i][1];
	}

	// Centers
	centerx /= (float)n;
	centery /= (float)n;

	/*calcul de la variance de x*/
	variancex = 0;
	for(i=0; i<n; i++)
	variancex += (mesures[i][0] - centerx)*(mesures[i][0] - centerx);
	variancex /= (float)n;

	/*calcul de la covariance entre x et y*/
	covariance = 0;
	for(i=0; i<n; i++)
	covariance += (mesures[i][0] - centerx)*(mesures[i][1] - centery);
	covariance /= (float)n;

    // Return raport inertia
	return covariance/variancex;
}
//---------------------------------------------------------------------------
// Calcul des exposants d'Holder sur les valeurs de scores
int Holder(int NbrNodes, CNode **N, float *Exp)
{
	int     i, j, NbrMesures, Count;
	float **Regression, MesureMax;

	Regression = AllocateF(NbrNodes, 2);
	MesureMax  = N[0]->Coss - N[NbrNodes-1]->Coss;

	Count = 0;

	for(i=0; i<NbrNodes; i++)
	{
		NbrMesures = floor(log((float)(NbrNodes-i))/log(2.0f));
		for(j=NbrMesures; j>0; j--)
		{
			if((N[i]->Coss - N[i+(int)pow(2.0f, j)-1]->Coss)/MesureMax < ZERO)
				NbrMesures--;
			else
			{
				Regression[NbrMesures-j][1] =  log((N[i]->Coss - N[i+(int)pow(2.0f, j)-1]->Coss)/MesureMax);
				Regression[NbrMesures-j][0] = -log(2.0f)*(NbrMesures-j);
			}
		}

		if(NbrMesures>1)
		{
			Exp[Count] = LineairRegression(Regression, NbrMesures);

			if (Exp[Count] > 1.0f) Exp[Count] = 1.0f;

			Count++;
		}
	}

	// Librer la mmoire
	for(i=0; i<NbrNodes; i++)
	delete[] Regression[i];
	delete[] Regression;

	return Count;
}
//---------------------------------------------------------------------------
// Sparation en deux groupes des valeurs de cossimilarite
// par calcul du minimum d'inertie intraclasse mais uniquement aux points
// d'exposants d'Holder faibles
int SeparateNodes1(CNode **N, int Nbr, float *Exp, int NbrExp)
{
	int    i, j, k;
	int    Nmin;
	float  Imin;
	int    L1, L2;
	float  M0, M1, M2;
	float  V0, V1, V2;

	Nmin = 1;
	Imin = DEF_MAX;

	for(i=1; i<NbrExp; i++)
	{
		if(i > Nbr/2) break;

		if((Exp[i-1] <= SEUIL) && (Exp[i] > SEUIL))
		{
			L1 = i;
			L2 = Nbr - L1;

			M1 = 0.0f;
			for(j=0; j<L1; j++)
			M1 += N[j]->Coss;
			M1 /= (float)L1;

			M2 = 0.0f;
			for(j=L1; j<Nbr; j++)
			M2 += N[j]->Coss;
			M2 /= (float)L2;

			V1 = 0.0f;
			for(j=0; j<L1; j++)
			V1 += pow(N[j]->Coss - M1, 2.0f);

			V2 = 0.0f;
			for(j=L1; j<Nbr; j++)
			V2 += pow(N[j]->Coss - M2, 2.0f);

			if (V1+V2 < Imin)
			{
				Imin = V1+V2;
				Nmin = L1;
			}
		}
	}

	return Nmin;
}
//---------------------------------------------------------------------------
// Separation en deux groupes des valeurs de dissimilarite
// par calcul du minimum d'inertie intraclasse
int SeparateNodes2(CNode **N, int Nbr)
{
	int   i, j, k;
	int   Nmin;
	float Imin;
	int   L1, L2;
	float M1, M2;
	float V1, V2;

	Nmin = 1;
	Imin = DEF_MAX;

	for(i=1; i<Nbr; i++)
	{
		L1 = i;
		L2 = Nbr - L1;

		M1 = 0.0f;
		for(j=0; j<L1; j++)
		M1 += N[j]->Coss;
		M1 /= (float)L1;

		M2 = 0.0f;
		for(j=L1; j<Nbr; j++)
		M2 += N[j]->Coss;
		M2 /= (float)L2;

		V1 = 0.0f;
		for(j=0; j<L1; j++)
		V1 += pow(N[j]->Coss - M1, 2.0f);

		V2 = 0.0f;
		for(j=L1; j<Nbr; j++)
		V2 += pow(N[j]->Coss - M2, 2.0f);

		if (V1+V2 < Imin)
		{
			Imin = V1+V2;
			Nmin = L1;
		}
	}
	return Nmin;
}

