
#include "Utils.hpp"
//---------------------------------------------------------------------------
// Insertion sorting
static void InsertionSort(CNode **D, int size)
{
	int i, j;
	CNode *t;

    for(i=1; i<size; ++i)
    {
		t = D[i];
        j = i;

		while((j > 0) && (t->Coss > D[j-1]->Coss))
        {
			D[j] = D[j-1];
            --j;
        }
		D[j] = t;
    }
}
//---------------------------------------------------------------------------
// Partition and get the median
static int Partition(CNode **D, int f, int l, const float& pivot)
{
    int i=f-1;
	int j=l+1;

    while(true)
    {
		while(pivot > D[--j]->Coss);
		while(D[++i]->Coss > pivot);

        if(i<j)
			SWAP(D[i], D[j]);
        else
            return j;
    }
}
//---------------------------------------------------------------------------
// Sort a partition
static void Sort(CNode **D, int f, int l)
{
	int m;
	float v1, v2, v3;
	float median;

	while(f+16 < l)
    {
		v1 = D[f]->Coss;
		v2 = D[l]->Coss;
		v3 = D[(f+l)/2]->Coss;
        median = v1<v2 ? (v3<v1 ? v1 : (v2<v3 ? v2 : v3)) : (v3<v2 ? v2 : (v1<v3 ? v1 : v3));
        m = Partition(D, f, l, median);
        Sort(D, f, m);
        f = m+1;
    }
}
//---------------------------------------------------------------------------
// Quick sorting with detection of the best median
void SortNodes(CNode **D, int Nbr)
{
	Sort(D, 0, Nbr-1);
	InsertionSort(D, Nbr);
}
//---------------------------------------------------------------------------
// Quick sort distances
void QuickSort(CList *L, int n)
{
	const   int M=7, NSTACK=50;
	int  i, ir, j, k, jstack=-1, l=0;
	int *istack;
	CList   a;
	
	istack = AllocateI(NSTACK);
	ir = n-1;

	for (;;) 
	{
		if (ir-l < M) 
		{
			for (j=l+1; j<=ir; j++) 
			{
				a = L[j];
				for (i=j-1; i>=l; i--) 
				{
					if (L[i].D >= a.D) break;
					L[i+1] = L[i];
				}
				L[i+1] = a;
			}

			if (jstack < 0) break;
			ir = istack[jstack--];
			l  = istack[jstack--];
		} 
		else 
		{
			k = (l+ir) >> 1;
			SWAP(L[k], L[l+1]);

			if (L[l].D < L[ir].D)
			{
				SWAP(L[l], L[ir]);
			}

			if (L[l+1].D < L[ir].D)
			{
				SWAP(L[l+1], L[ir]);
			}

			if (L[l].D < L[l+1].D)
			{
				SWAP(L[l], L[l+1]);
			}

			i = l+1;
			j = ir;
			a = L[l+1];

			for (;;)
			{
				do i++; while (L[i].D > a.D);
				do j--; while (L[j].D < a.D);
				if (j < i) break;
				SWAP(L[i], L[j]);
			}

			L[l+1]  = L[j];
			L[j]    = a;
			jstack += 2;

			if (jstack >= NSTACK) 
			{
				printf("NSTACK too small in sort.");
				exit(EXIT_FAILURE);
			}

			if (ir-i+1 >= j-l) 
			{
				istack[jstack]   = ir;
				istack[jstack-1] = i;
				ir = j-1;
			} else 
			{
				istack[jstack]   = j-1;
				istack[jstack-1] = l;
				l=i;
			}
		}
	}
}
