//---------------------------------------------------------------------------
//
#include "Utils.hpp"
//
// Substitution matrix
//---------------------------------------------------------------------------

float   SUB_MAT1[SYMBOLS+1][SYMBOLS+1];
float   SUB_MAT2[SYMBOLS+1][SYMBOLS+1];
float   DEF_MAX  =   3.4E+38f;					// valeur trs grande
float   DEF_MIN  =  -3.4E+38f;					// valeur trs petite
float   GapEX    =   1.0f;						// Gap extension penalty
float   GapOP    =  10.0f;						// Gap open penalty
float   SbScr    =   0.0f;						// Minimum substitution score allowed for matching
int     MaxAllow = 500;							// Default maximum number of sequences allowed

//---------------------------------------------------------------------------
// Processing message error
void SendMsg(char *Msg, char *Code)
{
	// Format the message
	char Buff[MAXLEN];
	sprintf(Buff, "%s (ERROR CODE %s)", Msg, Code);

	switch (VERS)
	{
		case EXEC:	printf("%s\n", Buff); break;
		case HTML:	printf("Content-type: text/html\n\n");
					printf("<h2>%s</h2>\n", Buff);
					break;
		default:
		{
			printf("%s (CODE ERROR %s)", FATALERR, "55");
			exit(EXIT_FAILURE);
		}
	}
}
//---------------------------------------------------------------------------
// Get residue index from its char
char Index(int Symbol)
{
	switch  (Symbol)
	{
		case 'A' : return  0;
		case 'B' : return  1;
		case 'C' : return  2;
		case 'D' : return  3;
		case 'E' : return  4;
		case 'F' : return  5;
		case 'G' : return  6;
		case 'H' : return  7;
		case 'I' : return  8;
		case 'K' : return  9;
		case 'L' : return 10;
		case 'M' : return 11;
		case 'N' : return 12;
		case 'P' : return 13;
		case 'Q' : return 14;
		case 'R' : return 15;
		case 'S' : return 16;
		case 'T' : return 17;
		case 'V' : return 18;
		case 'W' : return 19;
		case 'X' : return 20;
		case 'Y' : return 21;
		case 'Z' : return 22;
		case '-' : return 23;
		default:
		{
			SendMsg(FATALERR, "052");
			exit(EXIT_FAILURE);
		}
	}
}
//---------------------------------------------------------------------------
// Get residue char from its index
char Char(char Index)
{
	switch  (Index)
	{
		case  0 : return 'A';
		case  1 : return 'B';
		case  2 : return 'C';
		case  3 : return 'D';
		case  4 : return 'E';
		case  5 : return 'F';
		case  6 : return 'G';
		case  7 : return 'H';
		case  8 : return 'I';
		case  9 : return 'K';
		case 10 : return 'L';
		case 11 : return 'M';
		case 12 : return 'N';
		case 13 : return 'P';
		case 14 : return 'Q';
		case 15 : return 'R';
		case 16 : return 'S';
		case 17 : return 'T';
		case 18 : return 'V';
		case 19 : return 'W';
		case 20 : return 'X';
		case 21 : return 'Y';
		case 22 : return 'Z';
		case 23 : return '-';
		default:
		{
			SendMsg(FATALERR, "053");
			exit(EXIT_FAILURE);
		}
	}
}
//---------------------------------------------------------------------------
// Get HTML parameter value
char *HTMLParameter(char *Buff, char *Bry, char *Par)
{
	char *ptr1;
	char *ptr2;
	static char Tmp1[MAXLEN];
	static char Tmp2[MAXLEN];

	// Prepare the parameter for which to get the value
	strcpy(Tmp1, "name=\"");
	strcat(Tmp1, Par);
	strcat(Tmp1, "\"");
	strcat(Tmp1, "\0");

	// Get for the parameter name
	if ((ptr1 = strstr(Buff, Tmp1)) == NULL) goto Error;
	ptr1 += strlen(Tmp1);

	// Remove hidden chars
	while (*ptr1 == char(10) || *ptr1 == char(13)) ptr1++;

	// Get the right limit of the parameter value
	if ((ptr2 = strstr(ptr1, Bry)) == NULL) goto Error;

	// Remove hidden chars
	while (*(ptr2-1) == char(10) || *(ptr2-1) == char(13)) ptr2--;

	// Get the parameter value
	strncpy(Tmp2, ptr1, ptr2-ptr1);

	// Add null char at the end
	Tmp2[ptr2-ptr1] = NULL;

	// Return the parameter value
	return Tmp2;

	// Return an error
	Error:
	{
		// Send error message
		SendMsg(FATALERR, "056");

		// Exit the program
		exit(EXIT_FAILURE);
	}
}
//---------------------------------------------------------------------------
// Get subfamily name of subfamily number
char *FileName(int Group, char *ext)
{
	char *str = AllocateC(MAXLEN);

	if (Group == 0)
		sprintf(str, "File.%s", ext);
	else
		sprintf(str, "File_%d.%s", Group, ext);

	return str;
}
//---------------------------------------------------------------------------
// Test if the input file is in FASTA format
bool FastaFormat(FILE *file, char Msg[MAXLEN])
{
	int   i, j;
	int   Count=0, nuc=1;
	char  Buff[MAXLEN];
	char  Name[MAXLEN];

	// On vrifie le format du fichier
	while (!feof(file))
	{
		// read the current line from the input file
		if (fgets(Buff, MAXLEN, file) == NULL) continue;

		// Trim special characters
		strtrim(Buff);

		// Remove the terminating character
		if (Buff[strlen(Buff)-1] == DEF_NL || Buff[strlen(Buff)-1] == DEF_CR) Buff[strlen(Buff)-1] = NULL;

		for (i=0; i<strlen(Buff); i++)
		{
			if ((Buff[i]) == '.' || (Buff[i]) == '~') Buff[i] = '-';
			if ((Buff[i]) == '*' || (Buff[i]) == 'U') Buff[i] = 'X';
		}

		if (Buff[0] == '>')
		{
			if (nuc == 0)
			{
				sprintf(Buff, "Input error in FASTA file !!!, empty sequence (%s)", Name);
				return false;
			}

			Count++;
			nuc = 0;

            // Get the name of the sequence
			strcpy(Name, Buff);
		}
		else
		{
			for (i=0; i<strlen(Buff); i++)
			{
				if (Buff[i] == '#' || Buff[i] == ' ')
					continue;
				else
					nuc++;

				if (Index((char)toupper(Buff[i])) < 0)
				{
					sprintf(Msg, "Input error in FASTA file !!!, invalid character (%c) in sequence: %s", Buff[i], Name);
					return false;
				}
			}
		}
	}

	if (nuc == 0)
	{
		strcpy(Msg, "Input error in FASTA file !!!, empty sequence");
		return false;
	}
	else
	if (Count == 0)
	{
		strcpy(Msg, "Input error in FASTA file !!!, no sequences");
		return false;
	}
	else
		return true;
}
//---------------------------------------------------------------------------
// Pearson correlation coefficient
void Pearson(float **Mat, int Count)
{
	int     i, j, k;
	float  *mean, *sxx;
	float **sxy;

	// Allocation mmoire
	sxy  = AllocateF(Count, Count);
	mean = AllocateF(Count);
	sxx  = AllocateF(Count);

	for (i=0; i<Count; i++)
	{
		mean[i] = 0.0f;
		sxx[i]  = 0.0f;

		for (j=0; j<Count; j++)
		{
			mean[i] += Mat[i][j];
			sxx[i]  += Mat[i][j] * Mat[i][j];

			sxy[i][j] = 0.0f;

			for (k=0; k<Count; k++)
			sxy[i][j] += Mat[i][k] * Mat[j][k];
		}
		mean[i] /= Count;
	}

	// Compute Pearson's correlation coefficient
	for (i=0; i<Count; i++)
	for (j=i; j<Count; j++)
	Mat[i][j] = Mat[j][i] = 1.0f - (sxy[i][j] - mean[i]*mean[j]*Count) / sqrt((sxx[i] - mean[i]*mean[i]*Count) * (sxx[j] - mean[j]*mean[j]*Count));

	// Libration de la mmoire
	for (i=0; i<Count; i++)
	delete[] sxy[i];
	delete[] sxy;
	delete[] mean;
	delete[] sxx;
}
//---------------------------------------------------------------------------
// Update the belonging of nodes
void UpdateNodes(CNode *N, CNode **L, int Group)
{
	if (N->Size == 1)
	{
		L[N->Num]->Group = Group;
	}
	else
	{
		UpdateNodes(N->L, L, Group);
		UpdateNodes(N->R, L, Group);
	}
}
//---------------------------------------------------------------------------
// SMS matching score
float Match1(CSeq *S1, CSeq *S2, int Min)
{
	int   i, j, k;
	int   Diag;
	float Result = 0.0f;
	float Score;

	for (i=0; i<S1->Length; i++)
	{
		Score = 0.0f;
		Diag  = 0;

		for (k=i, j=0; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j])
			{
				Diag++;
				Score += SUB_MAT1[S1->Res[k]][S2->Res[j]];
			}
			else
			{
				if (Diag > Min) Result += Score;

				Score = 0.0f;
				Diag  = 0;
			}
		}
		if (Diag > Min) Result += Score;
	}

	for (i=1; i<S2->Length; i++)
	{
		Score = 0.0f;
		Diag  = 0;

		for (k=0, j=i; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j])
			{
				Diag++;
				Score += SUB_MAT1[S1->Res[k]][S2->Res[j]];
			}
			else
			{
				if (Diag > Min) Result += Score;

				Score = 0.0f;
				Diag  = 0;
			}
		}
		if (Diag > Min) Result += Score;
	}
	return Result / MAX2(S1->Length, S2->Length);
}
//---------------------------------------------------------------------------
// SMS matching score
float Match2(CSeq *S1, CSeq *S2, int Min)
{
	int   i, j, k;
	int   Mach, Miss;
	float Result = 0.0f;
	float Score;

	for (i=0; i<S1->Length; i++)
	{
		Score = 0.0f;
		Mach  = 0;
		Miss  = 0;

		for (k=i, j=0; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j]) Mach++;
			else
			if (SUB_MAT1[S1->Res[k]][S2->Res[j]] > SbScr) Miss++;

			if (Miss > Min || SUB_MAT1[S1->Res[k]][S2->Res[j]] <= SbScr)
			{
				if (Mach >= Min) Result += Score;

				Score = 0.0f;
				Mach  = 0;
				Miss  = 0;
			}
			else Score += SUB_MAT1[S1->Res[k]][S2->Res[j]];
		}
		if (Mach > Min && Miss < Min) Result += Score;
	}

	for (i=1; i<S2->Length; i++)
	{
		Score = 0.0f;
		Mach  = 0;
		Miss  = 0;

		for (k=0, j=i; k<S1->Length && j<S2->Length; k++, j++)
		{
			if (S1->Res[k] == S2->Res[j]) Mach++;
			else
			if (SUB_MAT1[S1->Res[k]][S2->Res[j]] > SbScr) Miss++;

			if (Miss > Min || SUB_MAT1[S1->Res[k]][S2->Res[j]] <= SbScr)
			{
				if (Mach >= Min) Result += Score;

				Score = 0.0f;
				Mach  = 0;
				Miss  = 0;
			}
			else Score += SUB_MAT1[S1->Res[k]][S2->Res[j]];
		}
		if (Mach > Min && Miss < Min) Result += Score;
	}
	return Result / MAX2(S1->Length, S2->Length);
}

