//---------------------------------------------------------------------------
//
#include <stdio.h>
#include <io.h>
#include <windows.h>
#include <math.h>
#include <ctype.h>
#include <fcntl.h>
#include <time.h>
#include <direct.h>

#define SYMBOLS     23							// Amino acids number
#define ALPHA      256							// alphabet size
#define MINDIAG      3							// Minimum length of matched motifs
#define ZERO         1e-7						// valeur proche du zero
#define MAXLEN    1024							// length of taxon names
#define SEUIL        0.5
#define LOADERR     "Loading error !!!"
#define FATALERR    "Fatal error !!!"
#define IETRATION  100
#define LOCAL		10

#define MIN2(a,b)		(a < b ? a : b)
#define MAX2(a,b)		(a > b ? a : b)
#define MAX3(a,b,c)		(a > MAX2(b,c) ? a : MAX2(b,c))
#define MAX4(a,b,c,d)	(a > MAX3(b,c,d) ? a : MAX3(b,c,d))
#define SIGN(a,b)		(b >= 0.0f ? fabs(a) : -fabs(a))

#define LINE_CLUSTAL 50
#define LINE_FASTA   70
#define NAME_LENGTH  20

#define IP_CLUSTAL	 1
#define IP_FASTA	 2
#define IP_NATIVE	 3

#define IP_LEFT	     1
#define IP_RIGHT	 2

#define IP_DOUBLEI   1
#define IP_DOUBLEO   2

#define IP_LEAFN	 1
#define IP_LEAFI	 2

#define IP_BLOSUM62  1
#define IP_PAM250    2

#define IP_ALIGNALL  1
#define IP_ALIGNSUB  2

#define IP_REFWI     1
#define IP_REFNO     2

#define IP_LOCALALY  1
#define IP_LOCALALN  2

#define ALIGNER1     1
#define ALIGNER2     2
#define ALIGNER3     3

#define DEF_TB      '\t'
#define DEF_CR      '\r'
#define DEF_NL      '\n'

#define HTML 1
#define EXEC 2
#define VERS EXEC

extern float SUB_MAT1[SYMBOLS+1][SYMBOLS+1];			// Substitution scores matrix
extern float SUB_MAT2[SYMBOLS+1][SYMBOLS+1];			// Substitution scores matrix

extern float DEF_MAX;							// valeur trs grande
extern float DEF_MIN;							// valeur trs petite

extern float GapEX;								// Gap extension penalty
extern float GapOP;								// Gap open penalty
extern float SbScr;								// Minimum substitution score allowed for matching
extern int   MaxAllow;  						// Maximum number of sequences allowed
//---------------------------------------------------------------------------
// Structure d'un somet
typedef struct CVertx
{
	int     Num;								// Vertex number id
	int     Group;								// Corresponding group
	int     Size;								// Size of current tree
	CVertx *Next;								// Next vertex
} CVertx;
//---------------------------------------------------------------------------
// Structure de la liste des distances
typedef struct CList
{
	int   I;
	int   J;
	float D;
} CList;
//---------------------------------------------------------------------------
// Structure : One residue
typedef struct CRes
{
	int Indx;									// Num of residue
	int Twin;									// Matched residue
	int Type;
	int Seq1;									// Sequence index
	int Res1;									// Residue index
	int Seq2;									// Sequence index
	int Res2;									// Residue index
} CRes;
//---------------------------------------------------------------------------
// Structure : One residue
typedef struct CMot
{
	int   Nbr;									// Sequence index
	CRes *Res; 									// Residue1 index
} CMot;
//---------------------------------------------------------------------------
// Structure : Une squence
typedef struct CSeq
{
	int    Num;									// Sequence index
	int    Length;								// Length of the sequence
	float  Weight;								// Weight of the sequence
	char   Name1[MAXLEN];						// Sequence name
	char   Name2[MAXLEN];						// Sequence name token
	char  *Res;									// Amino acid
} CSeq;
//---------------------------------------------------------------------------
// Structure d'un noeud
typedef struct CNode
{
	int    Num;                            		// Node number
	int    Size;	                        	// size of current cluster
	int    Group;								// Belonging group
	int    Length;								// Length of the alignment
	bool   Alig;								// If aligned
	float  Dist;                           		// Distance de son parent
	float  Weight;								// The weight
	float  Coss;								// Cossimilarity
	float  Score;								// Alignment score
	CSeq **S;									// Aligned sequences
	CNode *L;                              		// Left node
	CNode *R;                              		// Right node
	CNode *T;									// Tail node
} CNode;
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
// Routines declarations
//
//---------------------------------------------------------------------------
// Swap two variables
template<class T>
inline void SWAP(T &a, T &b)	{T t=a; a=b; b=t;}
//---------------------------------------------------------------------------
// Cosin product
inline float Cosin(float *v1, float *v2, int Dim)
{
	float m0=0.0f;
	float m1=0.0f;
	float m2=0.0f;

	for (int i=0; i<Dim; i++)
	{
		m0 += v1[i] * v2[i];
		m1 += v1[i] * v1[i];
		m2 += v2[i] * v2[i];
	}

	if (m1 == 0.0f || m2 == 0.0f)
		return 0.0f;
	else
		return m0/sqrt(m1*m2);
}
//---------------------------------------------------------------------------
// Cosin product
inline float Inner(float *v1, float *v2, int Dim)
{
	float Result=0.0f;

	for (int i=0; i<Dim; i++)
	Result += v1[i] * v2[i];

	return Result;
}
//---------------------------------------------------------------------------
// Vector module
inline float Norme(float *v, int D)
{
	float Result = 0.0f;

	for (int i=0; i<D; i++)
	Result += v[i]*v[i];

	return sqrt(Result);
}
//---------------------------------------------------------------------------
// Compute (a^2 + b^2)^1/2 without destructive undeflow or overflow
inline float PYTH(float a, float b)
{
	float absa = fabs(a);
	float absb = fabs(b);

	if (absa > absb)
		return absa * sqrt(1.0f + (absb/absa)*(absb/absa));
	else
		return (absb == 0.0f ? 0.0f : absb * sqrt(1.0f + (absa/absb)*(absa/absb)));
}
//---------------------------------------------------------------------------
// Trim a string
inline void strclean(char *Str)
{
	for (unsigned int i=0; i<strlen(Str); i++)
	if ((Str[i] == char(10)) || (Str[i] == char(13)))
	{
		for (unsigned int j=i; j<strlen(Str)-1; j++)
		Str[j] = Str[j+1];
		Str[strlen(Str)-1] = NULL;

		// Avoid go to the next char
		i--;
	}
}
//---------------------------------------------------------------------------
// Trim a string
inline char *strtrim(char *Str)
{
	for (unsigned int i=0; i<strlen(Str); i++)
	if (Str[i] == DEF_NL || Str[i] == DEF_CR)
	{
		for (unsigned int j=i; j<strlen(Str)-1; j++)
		Str[j] = Str[j+1];
		Str[strlen(Str)-1] = NULL;

		// Avoid go to the next char
		i--;
	}

	// Return the trimed str
	return Str;
}
//---------------------------------------------------------------------------
// 1D boolean allocation memory
inline bool *AllocateB(int m)
{
	bool *A = new bool[m];
	memset(A, false, m*sizeof(bool));

	return A;
}
//---------------------------------------------------------------------------
// 1D integer allocation memory
inline int *AllocateI(int m)
{
	int *A = new int[m];
	memset(A, 0, m*sizeof(int));

	return A;
}
//---------------------------------------------------------------------------
// 2D integer allocation memory
inline int **AllocateI(int m, int n)
{
	int **A = new int*[m];
	for (int i=0; i<m; i++)
	{
		A[i] = new int[n];
		memset(A[i], 0, n*sizeof(int));
	}

	return A;
}
//---------------------------------------------------------------------------
// 1D float allocation memory
inline float *AllocateF(int m)
{
	float *A = new float[m];
	memset(A, 0, m*sizeof(float));

	return A;
}
//---------------------------------------------------------------------------
// 1D float allocation memory
inline CMot *AllocateM(int m)
{
	CMot *A = new CMot[m];
	for (int i=0; i<m; i++)
	{
		A[i].Nbr = 0;
		A[i].Res = NULL;
	}

	return A;
}
//---------------------------------------------------------------------------
// 2D float allocation memory
inline float **AllocateF(int m, int n)
{
	float **A = new float*[m];
	for (int i=0; i<m; i++)
	{
		A[i] = new float[n];
		memset(A[i], 0, n*sizeof(float));
	}

	return A;
}
//---------------------------------------------------------------------------
// 1D char allocation memory
inline char *AllocateC(int m)
{
	char *A = new char[m];
	A[0] = NULL;

	return A;
}
//---------------------------------------------------------------------------
// Processing message error
void SendMsg(char *Msg, char *Code);
//---------------------------------------------------------------------------
// Get residue index from its char
char Index(int Symbol);
//---------------------------------------------------------------------------
// Get residue char from its index
char Char(char Index);
//---------------------------------------------------------------------------
// Get HTML parameter value
char *HTMLParameter(char *Buff, char *Bry, char *Par);
//---------------------------------------------------------------------------
// Compute (a^2 + b^2)^1/2 without destructive undeflow or overflow
float PYTH(float a, float b);
//---------------------------------------------------------------------------
// Get subfamily name of subfamily number
char *FileName(int Group, char *ext);
//---------------------------------------------------------------------------
// Calculate the eigen vectors and eigen values of a square matrix
int Eigen(float **a, int L);
//---------------------------------------------------------------------------
// Singular value decomposition
void SVD(float **a, int m, int n);
//---------------------------------------------------------------------------
// Calcul des exposants d'Holder sur les valeurs de scores
int Holder(int NbrNodes, CNode **SortedNodes, float *Exp_Holder);
//---------------------------------------------------------------------------
// Sparation en deux groupes des valeurs de cossimilarite
// par calcul du minimum d'inertie intraclasse mais uniquement aux points
// d'exposants d'Holder faibles
int SeparateNodes1(CNode **N, int Nbr, float *Exp, int NbrExp);
//---------------------------------------------------------------------------
// Separation en deux groupes des valeurs de dissimilarite
// par calcul du minimum d'inertie intraclasse
int SeparateNodes2(CNode **N, int Nbr);
//---------------------------------------------------------------------------
// If is a subfamily node
bool SubGroupNode(CNode *N, float T);
//---------------------------------------------------------------------------
// Quick sort distances
void SortNodes(CNode **N, int Nbr);
//---------------------------------------------------------------------------
// Rechercher tous les noeuds
void FindNodes(CNode **Nodes, CNode *Node);
//---------------------------------------------------------------------------
// Pearson correlation coefficient
void Pearson(float **Mat, int Count);
//---------------------------------------------------------------------------
// Test if the input file is in FASTA format
bool FastaFormat(FILE *file, char Msg[MAXLEN]);
//---------------------------------------------------------------------------
// Quick sort distances
void QuickSort(CList *L, int n);
//---------------------------------------------------------------------------
// Update the belonging of nodes
void UpdateNodes(CNode *N, CNode **L, int Group);
//---------------------------------------------------------------------------
// SMS matching score
float Match1(CSeq *S1, CSeq *S2, int Min);
//---------------------------------------------------------------------------
// SMS matching score
float Match2(CSeq *S1, CSeq *S2, int Min);

